
// Dynamic Listbox data
var listdata = [ "checkpointed", "reviewed", "fun", "visual" ];

// Custom View tree data
var treedata = [
  [ "checkpointed", "breakdown" ],
  [ "checkpointed", "first draft" ],
  [ "checkpointed", "final" ],
  [ "reviewed", "guru" ],
  [ "reviewed", "rubbish" ],
  [ "fun", "cool" ],
  [ "guru", "cool" ]
];
        
// initialise all views
function init_views()
{
  var listbox = document.getElementById("dialog.keywords");
  var tree = document.getElementById("dialog.related");
  var view = null;

  listbox.myview    = new dynamicListBoxView();
  listbox.mybuilder = new dynamicListBoxBuilder(listbox);
  listbox.mybuilder.rebuild();

  view = new customTreeView(tree);
  tree.view = view;
}

window.addEventListener("load",init_views, true);

// Listbox view
function dynamicListBoxView() {}

dynamicListBoxView.prototype = {
  get rowCount ()
  {
    return listdata.length;
  },
  getItemText: function (index)
  {
    return listdata[index];
  }
};

// Listbox builder
function dynamicListBoxBuilder(listbox) { this.init(listbox); }

dynamicListBoxBuilder.prototype = {
  _listbox : null,
  init : function (listbox) {
    _listbox = listbox;
  },
  rebuild : function () {
    var rows, item;
    try { _listbox.myview; }
    catch (e) { throw("Dynamic Listbox not initialised. " + e); }

    try { _listbox.myview.rowCount; }
    catch (e) { throw("Dynamic Listbox has no view" + e); }

    while (_listbox.hasChildNodes())
      _listbox.removeChild(_listbox.lastChild);

    rows = _listbox.myview.rowCount;

    for (var i=0; i < rows; i++)
    {
      item = document.createElement("listitem");
      item.setAttribute("label", _listbox.myview.getItemText(i));
      _listbox.appendChild(item);
    }
  }
};

// tree view
function customTreeView(tree) {
  this.calculate();
  this._tree = tree;
}

customTreeView.prototype = {

  // ---  application specific features
  _relatedMatrix : null,
  _openTree : null,
  _viewMap : null,

  calculate : function () {
    this.calcRelatedMatrix();
    this.calcTopOfTree();
    this.calcViewMap();
  },

  calcRelatedMatrix : function () {
    this._relatedMatrix = {};
    var i = 0, r = this._relatedMatrix;
    while ( i < treedata.length )
    {
      if ( ! (treedata[i][0] in r) )
          r[treedata[i][0]] = {};
      if ( ! (treedata[i][1] in r) )
          r[treedata[i][1]] = {};

      r[treedata[i][0]][treedata[i][1]] = true;
      r[treedata[i][1]][treedata[i][0]] = true;
      i++;
    }
  },

  calcTopOfTree : function () {
    var i;
    this._openTree  = [];

    for (i=0; i < listdata.length; i++)
    {
      this._openTree[i] = { container : false,
                            open : false,
                            keyword : listdata[i],
                            kids : null,
                            level : 0
                          };
      if ( typeof(this._relatedMatrix[listdata[i]]) != "undefined" )
        this._openTree[i].container = true;
    }
  },

  calcViewMap : function () {
    this._viewMap = [];
    this.calcViewMapTreeWalker(this._openTree, 0);
  },

  calcViewMapTreeWalker : function(kids, level) {
    for (var i=0; i < kids.length; i++ )
    {
      this._viewMap.push(kids[i]);
      if ( kids[i].container == true && kids[i].open == true )
        this.calcViewMapTreeWalker(kids[i].kids, level + 1);
    }
  },

  // --- nsITreeView interface

  // methods the application needs
  get rowCount() {
    return this._viewMap.length;
  },

  getCellText: function(row, column) {
    return this._viewMap[row].keyword;
  },

  isContainer: function(index) {
    return this._viewMap[index].container;
  },

  isContainerOpen: function(index) {
    return this._viewMap[index].open;
  },

  isContainerEmpty: function(index) {
    var kids = this._viewMap[index].kids;
    if ( kids == null ) return false;        // isn't a container.
    return ( kids.length == 0);
  },

  getLevel: function(index) {
    return this._viewMap[index].level;
  },

  getParentIndex: function(index) {
    var level = this._viewMap[index].level;
    while ( --index >= 0 )
      if ( this._viewMap[index].level < level )
        return index;
    return -1;
  },

  hasNextSibling: function(index, after) {
    var level = this._viewMap[index].level;
    while ( ++index < this._viewMap.length )
    {
      if ( this._viewMap[index].level < level )
        return false;
      if ( this._viewMap[index].level == level && index > after )
       return true;
    }
    return false;
  },

  toggleOpenState: function(index) {
    var i = 0;
    var node = this._viewMap[index];
    if ( ! node.container )
      return;
    if ( node.open )
    {
      node.open = false;
      node.kids = null;
      i = index + 1;
      while ( this._viewMap[index].level > this._viewMap[i].level)
        i++;
      i = i - index;
    }
    else
    {
      node.open = true;
      node.kids = [];
      for (var key in this._relatedMatrix[node.keyword])
      {
        node.kids[i] = { container : false,
                           open : false,
                         keyword : key,
                         kids : null,
                         level : node.level + 1
                       };
        if ( typeof(this._relatedMatrix[key]) != "undefined" )
          node.kids[i].container = true;
        i++;
      }
    }
    this.calcViewMap();
    this._tree.treeBoxObject.rowCountChanged(index,i);
  },

  // methods the builder needs, but which do nothing.
  canDropBeforeAfter: function(index, before) { return false; },
  canDropOn: function(index) { return false; },
  cycleCell: function(row, column) {},
  cycleHeader: function(col, elem) {},
  drop: function(row, orientation) { return false; },
  getCellProperties: function(row, prop) {},
  getCellValue: function(row, column) {},
  getColumnProperties: function(column, elem, prop) {},
  getImageSrc: function(row, column) {},
  getProgressMode: function(row, column) {},
  getRowProperties: function(row, column, prop) {},
  isEditable: function(row, column) { return false; },
  isSeparator: function(index) { return false; },
  isSorted: function() { return false; },
  performAction: function(action) {},
  performActionOnCell: function(action, row, column) {},
  performActionOnRow: function(action, row) {},
  selectionChanged: function() {},
  setCellText: function(row, column, value) {},
  setTree: function(tree) {}
};
